using System;
using System.Collections.Generic;
using System.Text;
using Brume;

namespace TestBrume.Effects
{
    class FireEffect
    {
        private BrumeVector _vOffset;

        public BrumeVector Offset
        {
            get { return _vOffset; }
            set { _vOffset = value; }
        }

        private float _fMaxLifeTime = 600f;

        public float MaxLifeTime
        {
            get { return _fMaxLifeTime; }
            set { _fMaxLifeTime = value; }
        }


        public FireEffect(Brume.Brume pBrume, BrumeObject pObject, BrumeVector pOffset)
        {
            _brume = pBrume;
            _vOffset = pOffset;
            _boundObject = pObject;


            InitParticles();
        }


        public void Update(float fElapsedTime)
        {
            foreach (FireParticle particle in _particles)
            {
                particle.Update(fElapsedTime);

                if (particle.LifeTime < 0)
                {
                    InitParticle(particle);
                }


            }
        }

        public void Remove()
        {
            foreach (FireParticle particle in _particles)
            {
                particle.Remove();
            }


        }


        private void InitParticles()
        {
            for (int i = 0; i < _particles.Length; i++)
            {
                float x = (float)_random.NextDouble() / 3 - 0.5f;
                float z = (float)_random.NextDouble() / 4 - 0.12f;

                BrumeVector position = new BrumeVector();
                position.X = _boundObject.Pos.X + _vOffset.X + x;
                position.Y = _boundObject.Pos.Y + _vOffset.Y;
                position.Z = _boundObject.Pos.Z + +_vOffset.Z + z;

                BrumeVector direction = (_boundObject.Pos + _vOffset + new BrumeVector(1, 0, 0)) - position;
                direction.Normalize();

                _particles[i] = new FireParticle(_brume, position, direction, (float)_random.NextDouble() * _fMaxLifeTime);
                 
                
            }


        }

        /// <summary>
        /// Initializes the particle
        /// </summary>
        private void InitParticle(FireParticle particle)
        {
            float x = (float)_random.NextDouble() / 3   - 0.5f;
            float z = (float)_random.NextDouble() / 4  - 0.12f;

            BrumeVector position = new BrumeVector();
            position.X = _boundObject.Pos.X + _vOffset.X + x;
            position.Y = _boundObject.Pos.Y + _vOffset.Y ;
            position.Z = _boundObject.Pos.Z + +_vOffset.Z + z;

            particle.Direction = (_boundObject.Pos + _vOffset + new BrumeVector(1,0,0))  - position;
            particle.Direction.Normalize();

            particle.Pos = position;
            particle.LifeTime = (float)_random.NextDouble() * _fMaxLifeTime;

        }

        BrumeObject _boundObject = null;

        FireParticle[] _particles = new FireParticle[60];

        Random _random = new Random();

        Brume.Brume _brume = null;


    }
}
