using System;
using System.Collections.Generic;
using System.Text;
using Brume;

namespace TestBrume.Effects
{
    class FireParticle
    {

        private float _fLifeTime;

        public float LifeTime
        {
            get { return _fLifeTime; }
            set { _fLifeTime = value; }
        }

        private BrumeVector _vDirection;

        public BrumeVector Direction
        {
            get { return _vDirection; }
            set { _vDirection = value; }
        }



        public BrumeVector Pos
        {
            get { return _object.Pos; }
            set { _object.Pos = value; }
        }


        public FireParticle(Brume.Brume pBrume)
        {
            Initialize(pBrume);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FireParticle"/> class.
        /// </summary>
        /// <param name="pBrume">The p brume.</param>
        /// <param name="pLocation">The p location.</param>
        /// <param name="pDirection">The p direction.</param>
        /// <param name="pLifeTime">The p life time.</param>
        public FireParticle(Brume.Brume pBrume , BrumeVector pLocation , BrumeVector pDirection ,float pLifeTime  )
        {
            Initialize(pBrume);

            _object.Pos = pLocation;
            _vDirection = pDirection;
            _fLifeTime = pLifeTime;
        }

        private void Initialize(Brume.Brume pBrume)
        {
            _brume = pBrume;

            _object = new BrumeObject(pBrume, "particle");
            _object.Mesh = MeshLoader.GetPreloadedMesh("fire");
            _object.RenderStates.Texture[0] = pBrume.GetTexture("fire.bmp");
            _object.RenderStates.Lighted = false;

            _object.RenderStates.AlphaBlending = BrumeRenderParams.ALPHABLENDING_TEXTUREALPHA;
            _object.Size = 0.2f;
            _object.Orient(new BrumeVector(0, -1, 0), new BrumeVector(0, 0, -1));

        }




        /// <summary>
        /// Updates the particle
        /// </summary>
        /// <param name="pElapsedTime">The p elapsed time.</param>
        public void Update(  float pElapsedTime)
        {
            _object.Pos.X += _vDirection.X * pElapsedTime / _fMoveSpeed;
            _object.Pos.Y += _vDirection.Y * pElapsedTime / _fMoveSpeed;
            _object.Pos.Z += _vDirection.Z * pElapsedTime / _fMoveSpeed;

            _fLifeTime -= pElapsedTime;              

        }



        public void Remove()
        {
            _brume.RemoveObject(_object);
        }

        Brume.Brume _brume = null;
        BrumeObject _object = null;
        private float _fMoveSpeed = 500;



    }
}
