float4x4 matWorldViewProj;    // World * View * Projection matrix
float4x4 matWorld;            // world
float3 lightDir;              // light direction
float3 cameraPos;             // eye direction
float3 ambient;               // ambient intensity
float3 diffuse;               // diffuse intensity
float3 specular;              // specular intensity



struct VS_OUTPUT
{
    float4 Position   : POSITION;
    float3 Light : TEXCOORD0;
    float3 Normal : TEXCOORD1;
    float3 View : TEXCOORD2;
};


VS_OUTPUT SpecularVS( float4 vPos : POSITION, float3 Normal : NORMAL)
{
    VS_OUTPUT Output;
    
    Output.Position = mul(vPos, matWorldViewProj);
    Output.Normal = mul(Normal, matWorld);
    Output.Light = lightDir;

    float4 posWorld = mul(vPos, matWorld);	
    Output.View = cameraPos - posWorld;


    return Output;    
}


struct PS_OUTPUT
{
    float4 Color : COLOR0;  
};


PS_OUTPUT PhongPS(float3 Light : TEXCOORD0, float3 Normal : TEXCOORD1, float3 View : TEXCOORD2) 
{ 
    PS_OUTPUT Output;

    // phong specular    
    Output.Color = float4(0,0,0,0);
    
    float3 nLight = normalize(Light);
    float3 nNormal = normalize(Normal);
    float3 nView = normalize(View);
    
    float diff = saturate(dot(nNormal, nLight));
    float3 reflect = normalize(2 * diff * nNormal - nLight);

    float3 ambientDiffuse = ambient + diffuse * diff;
    
    Output.Color.xyz = ambientDiffuse + specular * pow(saturate(dot(reflect,nView)),8);
	
    return Output;
}


PS_OUTPUT BlinnPhongPS(float3 Light: TEXCOORD0, float3 Norm : TEXCOORD1, 
           float3 View : TEXCOORD2) : COLOR
{
    PS_OUTPUT Output;
    Output.Color = float4(0,0,0,0);
    
    // blinn-phong specular    
    float3 Normal = normalize(Norm);
    float3 lightDir = normalize(Light);
    float3 Half = normalize(lightDir + normalize(View));    

    float diff = saturate(dot(Normal, lightDir)); 

    float blinnSpecular = pow(saturate(dot(Normal, Half)), 25); 

    float3 ambientDiffuse = ambient + diffuse * diff;

    Output.Color.xyz = ambientDiffuse + specular * blinnSpecular; 
    
    return Output;
    
}

technique PhongShader
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SpecularVS( );
        PixelShader  = compile ps_2_0 PhongPS( );
    }

}

technique BlinnPhongShader
{
    pass P0
    {          
        VertexShader = compile vs_1_1 SpecularVS( );
        PixelShader  = compile ps_2_0 BlinnPhongPS( );
    }

}
